# -*- encoding: utf-8 -*-
"""
Copyright (c) 2019 - present AppSeed.us
"""

from django.conf import settings
from .util import *
import os
from django.apps import apps
from django.db import models

API_GENERATOR = {}
WEB_PATH = ""

try:
    API_GENERATOR = getattr(settings, 'API_GENERATOR') 
    WEB_PATH = getattr(settings, "WEB_GENERATOR_PATH")
except:     
    pass 

# For cross platform imports 
# https://stackoverflow.com/questions/6028000/how-to-read-a-static-file-from-inside-a-python-package

try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 `importlib_resources`.
    import importlib_resources as pkg_resources

def generate_enum():
    #for val in API_GENERATOR.values():
    for val in valid_models():        
        # extract model from import path
        app_name      = val.split('.')[0]
        model_name    = val.split('.')[-1]
        folder_path = os.path.join(WEB_PATH, "modules", to_filename(app_name), to_filename(model_name))
        file_path = os.path.join(folder_path, f"{to_filename(model_name)}.enum.ts")

        os.makedirs(folder_path, exist_ok=True)

        with open(file_path, 'w') as enum_file:
            enum_file.write("")

    return "generation"

def generate_model():
    # Import your base template for the model interface
    from . import web_model

    model_structure = pkg_resources.read_text(web_model, 'model_structure')
    # Example: model_structure might be something like:
    # "export interface {model_name} {{\n{fields}\n}}"

    for val in valid_models():
        app_name = val.split('.')[0]
        model_name = val.split('.')[-1]

        # Get the actual Django model class
        model_class = apps.get_model(app_name, model_name)

        # Build the TypeScript fields string
        ts_fields = []

        for field in model_class._meta.get_fields():
            # Skip reverse relations and auto-created fields
            if field.auto_created and not field.concrete:
                continue

            field_name = field.name
            field_class_name = field.__class__.__name__

            # Map Django field types to TypeScript types (extend as needed)
            DJANGO_TO_TS = {
                'CharField': 'string',
                'TextField': 'string',
                'EmailField': 'string',
                'URLField': 'string',
                'IntegerField': 'number',
                'FloatField': 'number',
                'BooleanField': 'boolean',
                'DateField': 'string | Date',
                'DateTimeField': 'string | Date',
                'ForeignKey': 'string',  # Typically the ID field
                'ManyToManyField': 'string[]',
            }

            ts_type = DJANGO_TO_TS.get(field_class_name, 'any')

            # Normal field
            ts_fields.append(f"    {field_name}: {ts_type};")

            # Add _detail field if ForeignKey
            if field_class_name == 'ForeignKey':
                related_model = field.related_model.__name__
                ts_fields.append(f"    {field_name}_detail: {related_model};")

        fields_str = "\n".join(ts_fields)

        # Format the model interface content
        generation = model_structure.format(
            model_name=model_name,
            fields=fields_str
        )

        # Define path to save model file
        folder_path = os.path.join(WEB_PATH, "modules", to_filename(app_name), to_filename(model_name))
        os.makedirs(folder_path, exist_ok=True)
        file_path = os.path.join(folder_path, f"{to_filename(model_name)}.model.ts")

        with open(file_path, 'w') as model_file:
            model_file.write(generation)

    return generation

def generate_service():
    from . import web_service
    # Load the TypeScript service template
    service_structure = pkg_resources.read_text(web_service, 'service_structure')

    for val in valid_models():
        app_name = val.split('.')[0]
        model_name = val.split('.')[-1]

        model_lower = model_name.lower()
        endpoint = f"{to_kebab_case(model_name)}s"  # Customize if needed

        # Fill in the service structure template
        generation = service_structure.format(
            model_name=model_name,
            model_name_lower=model_lower,
            endpoint=endpoint,
            model_file=f"{to_filename(model_name)}.model",
            app_name=app_name
        )

        # Output path: modules/{app_name}/{model_name}/{model_name}.service.ts
        folder_path = os.path.join(WEB_PATH, "modules",to_filename(app_name), to_filename(model_name))
        os.makedirs(folder_path, exist_ok=True)

        file_path = os.path.join(folder_path, f"{to_filename(model_name)}.service.ts")

        with open(file_path, 'w') as service_file:
            service_file.write(generation)

    return "Service generation complete."

def generate_detail():
    from . import web_detail
    detail_structure = pkg_resources.read_text(web_detail, 'detail_structure')

    for val in valid_models():
        app_name = val.split('.')[0]
        model_name = val.split('.')[-1]
        model_lower = model_name.lower()
        model_class = apps.get_model(app_name, model_name)
        model_var = model_lower

        fields = []
        for field in model_class._meta.get_fields():
            # skip reverse relations and M2M
            if field.auto_created and not field.concrete:
                continue
            if isinstance(field, models.ManyToManyField):
                continue
            fields.append(format_field_ts(field, model_var))

        fields_str = ",\n".join(fields)

        generation = detail_structure.format(
            model_name=model_name,
            model_name_lower=model_lower,
            model_file=to_filename(model_name),
            fields=fields_str,
            app_file_name=to_filename(app_name),
            model_file_name=to_filename(model_name)
        )

        folder_path = os.path.join(WEB_PATH, "modules", to_filename(app_name), to_filename(model_name), "components")
        os.makedirs(folder_path, exist_ok=True)
        file_path = os.path.join(folder_path, f"{to_filename(model_name)}.detail.tsx")

        with open(file_path, 'w') as f:
            f.write(generation)

    return "Detail component generation complete."

def generate_form():
    from . import web_form  # Your React form template as a text file

    # Load the form template with placeholders {model_name}, {model_lower}, {fields}
    form_template = pkg_resources.read_text(web_form, 'form_structure')

    def enum_to_label_value_array_name(field):
        # Assuming enums named like GenderType, UserType, etc. based on field name
        name = field.name.title().replace('_', '')
        return name

    def generate_form_field(field):
        name = field.name
        label = str(field.verbose_name).title() if field.verbose_name else name.title()

        # Validation rules
        rules = []
        if not getattr(field, 'blank', True) and not getattr(field, 'null', True):
            rules.append('{ required: true, message: "%s is required" }' % label)

        rules_str = ", ".join(rules)

        if isinstance(field, models.BooleanField):
            return f'''
            {{!props.{name} && (
      <Form.Item name="{name}" label="{label}" valuePropName="checked" rules={{[{rules_str}]}}>
        <Checkbox />
      </Form.Item>)}}'''

        elif field.choices:
            enum_name = enum_to_label_value_array_name(field)
            return f'''
            {{!props.{name} && (
      <Form.Item name="{name}" label="{label}" rules={{[{rules_str}]}}>
        <Select
          optionFilterProp="label"
          options={{enumToLabelValueArray({enum_name})}}
          placeholder="{label}"
        />
      </Form.Item>)}}'''

        elif isinstance(field, models.TextField):
            return f'''
            {{!props.{name} && (
      <Form.Item name="{name}" label="{label}" rules={{[{rules_str}]}}>
        <Input />
      </Form.Item>)}}'''

        elif 'phone' in name.lower():
            return f'''
            {{!props.{name} && (
      <Form.Item name="{name}" label="{label}" rules={{[{rules_str}]}}>
        <PhoneInput datatype="text" placeholder="{label}" />
      </Form.Item>)}}'''

        elif isinstance(field, (models.DateField, models.DateTimeField)):
            return f'''
            {{!props.{name} && (
      <Form.Item name="{name}" label="{label}" rules={{[{rules_str}]}}>
        <DatePicker style={{{{ width: "100%" }}}} />
      </Form.Item>)}}'''
        
        elif isinstance(field, (models.ForeignKey, models.OneToOneField)):
            return f'''
            {{!props.{name} && (
        <Form.Item name="{name}" label="{label}" rules={{[{rules_str}]}}>
            <{to_camel_case(field.related_model.__name__)}SearchInput detail={{data?.{name}_detail}} />
        </Form.Item>)}}'''

        else:
            # Default to Input
            return f'''
            {{!props.{name} && (
      <Form.Item name="{name}" label="{label}" rules={{[{rules_str}]}}>
        <Input placeholder="{label}" />
      </Form.Item>)}}'''

    for val in valid_models():
        app_label = val.split('.')[0]
        model_name = val.split('.')[-1]
        model_class = apps.get_model(app_label, model_name)

        # Collect fields JSX
        form_fields_jsx = []
        for field in model_class._meta.get_fields():
            # Skip reverse relations and m2m
            if field.auto_created and not field.concrete:
                continue
            if isinstance(field, models.ManyToManyField):
                continue
            form_fields_jsx.append(generate_form_field(field))

        fields_str = '\n'.join(form_fields_jsx)

        # Format full React form component string
        form_code = form_template.format(
            model_name=model_name,
            model_lower=model_name.lower(),
            fields=fields_str,
            app_file_name=to_filename(app_label),
            model_file_name=to_filename(model_name)
        )

        folder_path = os.path.join(WEB_PATH, 'modules', to_filename(app_label), to_filename(model_name), "components")
        os.makedirs(folder_path, exist_ok=True)
        file_path = os.path.join(folder_path, f'{to_filename(model_name)}.form.tsx')

        with open(file_path, 'w', encoding='utf-8') as f:
            f.write(form_code)

    return "Form components generated."

def generate_list():
    from . import web_list
    list_template = pkg_resources.read_text(web_list, 'list_structure')
    def generate_table_field(field):
        name = field.name
        title = name.replace('_', ' ').title()

        sorter = '{ multiple: 1 }'  # you can auto-increment for better sorting logic
        if field.choices:
            enum_name = name.title().replace('_', '')
            return f'''
        {{
        key: "{name}",
        title: "{title}",
        dataIndex: "{name}",
        render: (_, row) => getEnumName({enum_name}, row.{name}),
        sorter: {sorter},
        filters: enumToTextValueArray({enum_name}),
        }},'''

        elif isinstance(field, (models.DateTimeField, models.DateField)):
            return f'''
        {{
        key: "{name}",
        title: "{title}",
        dataIndex: "{name}",
        render: (_, row) => toDateAndTime(row.{name}),
        sorter: {sorter},
        }},'''

        elif isinstance(field, (models.TimeField)):
            return f'''
        {{
        key: "{name}",
        title: "{title}",
        dataIndex: "{name}",
        render: (_, row) => formatTo12Hour(row.{name}),
        sorter: {sorter},
        }},'''

        elif isinstance(field, (models.ForeignKey, models.OneToOneField)):
            return f'''
        {{
        key: "{name}",
        title: "{title}",
        dataIndex: "{name}",
        render: (_, row) => row.{name}_detail.name,
        filterDropdown: ({{setSelectedKeys, confirm }}) => (
          <div style={{{{ padding: 8 }}}}>
            <{to_camel_case(name)}SearchInput
              defaultOpen
              onChange={{(value) => {{
                setSelectedKeys(value ? [value] : []);
                confirm();
              }}}}
              style={{{{ width: 200 }}}}
              allowClear
            />
          </div>
        ),
        sorter: {sorter},
        }},'''

        else:
            return f'''
        {{
        key: "{name}",
        title: "{title}",
        dataIndex: "{name}",
        sorter: {sorter},
        }},'''

    for val in valid_models():
        app_name = val.split('.')[0]
        model_name = val.split('.')[-1]
        model_class = apps.get_model(app_name, model_name)
        # Then format the template
        columns_code = "\n".join(generate_table_field(f) for f in model_class._meta.get_fields() if f.concrete and not f.auto_created)

        list_code = list_template.format(
            model_name=model_name,
            model_lower=model_name.lower(),
            fields=columns_code,
            app_file_name=to_filename(app_name),
            model_file_name=to_filename(model_name)
        )

        folder_path = os.path.join(WEB_PATH, 'modules', to_filename(app_name), to_filename(model_name), "components")
        os.makedirs(folder_path, exist_ok=True)
        file_path = os.path.join(folder_path, f'{to_filename(model_name)}.list.tsx')

        with open(file_path, 'w', encoding='utf-8') as f:
            f.write(list_code)


def generate_search_input():
    from . import web_search
    # Load the TypeScript service template
    search_structure = pkg_resources.read_text(web_search, 'search_structure')

    for val in valid_models():
        app_name = val.split('.')[0]
        model_name = val.split('.')[-1]

        pascal_model = to_pascal_case(model_name)
        camel_model = model_name.lower()

        generation = search_structure.format(
            pascal_model=pascal_model,
            camel_model=camel_model,
            model_file_name=to_filename(model_name)
        )

        folder_path = os.path.join(WEB_PATH, "modules", to_filename(app_name), to_filename(model_name), "components")
        os.makedirs(folder_path, exist_ok=True)

        file_path = os.path.join(folder_path, f"{to_filename(model_name)}.search.tsx")

        with open(file_path, 'w') as search_file:
            search_file.write(generation)

    return "Search generation complete."

def generate_list_page():
    from . import web_list_page
    # Load the TypeScript service template
    list_page_structure = pkg_resources.read_text(web_list_page, 'list_page_structure')

    for val in valid_models():
        app_name = val.split('.')[0]
        model_name = val.split('.')[-1]

        generation = list_page_structure.format(
            app_file_name=to_filename(app_name),
            model_name=to_pascal_case(model_name),
            model_file_name=to_filename(model_name)
        )

        folder_path = os.path.join(WEB_PATH, "app", "admin", to_filename(app_name), to_filename(model_name) + "s")
        os.makedirs(folder_path, exist_ok=True)

        file_path = os.path.join(folder_path, "page.tsx")

        with open(file_path, 'w') as list_page_file:
            list_page_file.write(generation)

    return "Search generation complete."

def generate_detail_page():
    from . import web_detail_page
    # Load the TypeScript service template
    detail_page_structure = pkg_resources.read_text(web_detail_page, 'detail_page_structure')

    for val in valid_models():
        app_name = val.split('.')[0]
        model_name = val.split('.')[-1]

        generation = detail_page_structure.format(
            app_file_name=to_filename(app_name),
            model_name=to_pascal_case(model_name),
            model_file_name=to_filename(model_name)
        )

        folder_path = os.path.join(WEB_PATH, "app", "admin", to_filename(app_name), to_filename(model_name) + "s", f"[{to_filename(model_name)}_id]")
        os.makedirs(folder_path, exist_ok=True)

        file_path = os.path.join(folder_path, "page.tsx")

        with open(file_path, 'w') as list_page_file:
            list_page_file.write(generation)

    return "Search generation complete."

def generate_edit_page():
    from . import web_edit_page
    # Load the TypeScript service template
    list_page_structure = pkg_resources.read_text(web_edit_page, 'edit_page_structure')

    for val in valid_models():
        app_name = val.split('.')[0]
        model_name = val.split('.')[-1]

        generation = list_page_structure.format(
            app_file_name=to_filename(app_name),
            model_name=to_pascal_case(model_name),
            model_file_name=to_filename(model_name)
        )

        folder_path = os.path.join(WEB_PATH, "app", "admin", to_filename(app_name), to_filename(model_name) + "s", f"[{to_filename(model_name)}_id]", "edit")
        os.makedirs(folder_path, exist_ok=True)

        file_path = os.path.join(folder_path, "page.tsx")

        with open(file_path, 'w') as list_page_file:
            list_page_file.write(generation)

    return "Search generation complete."

def generate_create_page():
    from . import web_create_page
    # Load the TypeScript service template
    create_page_structure = pkg_resources.read_text(web_create_page, 'create_page_structure')

    for val in valid_models():
        app_name = val.split('.')[0]
        model_name = val.split('.')[-1]

        generation = create_page_structure.format(
            model_name=to_pascal_case(model_name),
            app_file_name=to_filename(app_name),
            model_file_name=to_filename(model_name)
        )

        folder_path = os.path.join(WEB_PATH, "app", "admin", to_filename(app_name), to_filename(model_name) + "s", "create")
        os.makedirs(folder_path, exist_ok=True)

        file_path = os.path.join(folder_path, "page.tsx")

        with open(file_path, 'w') as list_page_file:
            list_page_file.write(generation)

    return "Search generation complete."

def generate_app_menu():
    from . import web_menu
    # Load the TypeScript service template
    menu_structure = pkg_resources.read_text(web_menu, 'menu_structure')
    apps = {}

    for val in valid_models():
        app_name = val.split('.')[0]
        model_name = val.split('.')[-1]
        apps[app_name] = apps.get(app_name, "")
        apps[app_name] += f'''  {{
            key: "/admin/{to_filename(app_name)}/{to_filename(model_name)}s",
            icon: <UserOutlined />,
            label: <Link href="/admin/{to_filename(app_name)}/{to_filename(model_name)}s">{to_title_case(model_name)}</Link>,
        }},
        '''
    
    for app in apps:
        generation = menu_structure.format(
            app_name_pascal=to_pascal_case(app),
            app_file_name=to_filename(app),
            app_name_tilte=to_title_case(app),
            children=apps[app]
        )

        folder_path = os.path.join(WEB_PATH, "app", "admin", to_filename(app))
        os.makedirs(folder_path, exist_ok=True)

        file_path = os.path.join(folder_path, f"{to_filename(app)}.menu.tsx")

        with open(file_path, 'w') as menu_file:
            menu_file.write(generation)

    return "Menu generation complete."


# -*- encoding: utf-8 -*-
"""
Copyright (c) 2019 - present AppSeed.us
"""

from . import manager
from . import web

def generate_api():
    manager.generate_serializer_file()
    manager.generate_views_file()
    manager.generate_urls_file()
    manager.generate_admin_file()

def generate_web():
    web.generate_enum()
    web.generate_model()
    web.generate_service()
    web.generate_detail()
    web.generate_form()
    web.generate_list()
    web.generate_search_input()
    web.generate_create_page()
    web.generate_edit_page()
    web.generate_detail_page()
    web.generate_list_page()
    web.generate_app_menu()


# -*- encoding: utf-8 -*-
"""
Copyright (c) 2019 - present AppSeed.us
"""

import os, importlib, shutil

from django.core.management.base import BaseCommand
from django.db.utils import OperationalError
from django.conf import settings

from django_api_gen import generate_web
from django_api_gen import api

# Cross platform imports 
# https://stackoverflow.com/questions/6028000/how-to-read-a-static-file-from-inside-a-python-package

try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 `importlib_resources`.
    import importlib_resources as pkg_resources

class Command(BaseCommand):
    help = 'API Code generator'

    def add_arguments(self, parser):
        parser.add_argument(
            '-f', action='store_true', help='Forcing Generate API for all models (no checks).',
        )

    def handle(self, *args, **options):

        #################################################    
        # Validate settings

        # BASE_DIR -> ROOT or the project
        if not hasattr(settings, 'BASE_DIR'):
            print(f" > Err: 'BASE_DIR' not found in settings")
            print(f'   Hint: this variable point to the ROOT of the project]')
            return 

        # BASE_DIR -> ROOT or the project
        if not hasattr(settings, 'API_GENERATOR'):
            print(f" > Err: 'API_GENERATOR' not found in settings")
            return 

        #################################################    

        API_GENERATOR = getattr(settings, 'API_GENERATOR')

        valid_models = 0
        for val in API_GENERATOR.values():

            app_name      = val.split('.')[0]
            model_name    = val.split('.')[-1]
            model_import  = val.replace('.'+model_name, '')             

            models       = importlib.import_module( model_import )
            model        = None 

            try:
                model = getattr(models, model_name)
                model_exists = True
            except:
                print(f' > Warn: Model [' + model_name + '] not defined in [' + val.replace(('.' + model_name),'') + ']' )
                continue

            try:
                model.objects.last()
            except:
                print(f' > Warn: [' + val + '] model not migrated in DB.' )
                continue
    
            valid_models += 1

        if 0 == valid_models:
            print(f" > Error: All Models defined in CFG are INVALID")
            print(f"      |-- (nothing is generated)")
            return             

        #################################################    
        # User Confirmation

        print(f" > Configuration looks good.")
        print(f"   !!! The API folder will be overwritten !!! ")

        if options['f']:

            print(f" > Generate API (forcing mode).")

        else:
            answer = input("Continue? [Y/n]")

            if answer.upper() not in ["Y", "YES"]:
                print(f"API generation cancelled.")
                return 

        #################################################    
        # API folder - deleted at each cycle
        
        API_DIR        = os.path.join( settings.BASE_DIR, 'api' )
        API_FILE_INIT  = os.path.join( API_DIR, '__init__.py' )
        API_FILE_VIEWS = os.path.join( API_DIR, 'views.py' )
        API_FILE_URLS  = os.path.join( API_DIR, 'urls.py'  )
        API_FILE_SERIZ = os.path.join( API_DIR, 'serializers.py' )

        try:
            shutil.rmtree( API_DIR )
        except:
            pass

        # create API dir (all cases)
        # os.mkdir( API_DIR )         

        # API_FILE_INIT_content  = pkg_resources.read_text(api, '__init__.py'    )
        # API_FILE_VIEWS_content = pkg_resources.read_text(api, 'views.py'       )
        # API_FILE_URLS_content  = pkg_resources.read_text(api, 'urls.py'        )
        # API_FILE_SERIZ_content = pkg_resources.read_text(api, 'serializers.py' )

        # with open( API_FILE_INIT, 'w') as API_FILE_INIT_py:
        #     API_FILE_INIT_py.write( API_FILE_INIT_content )

        # with open( API_FILE_VIEWS, 'w') as API_FILE_VIEWS_py:
        #     API_FILE_VIEWS_py.write( API_FILE_VIEWS_content )

        # with open( API_FILE_URLS, 'w') as API_FILE_URLS_py:
        #     API_FILE_URLS_py.write( API_FILE_URLS_content )            

        # with open( API_FILE_SERIZ, 'w') as API_FILE_SERIZ_py:
        #     API_FILE_SERIZ_py.write( API_FILE_SERIZ_content )    

        # Proceed with the SQL part
        generate_web()
        print(f"API successfully generated")

        # exit
        return 



# -*- encoding: utf-8 -*-
"""
Copyright (c) 2019 - present AppSeed.us
"""

import os, importlib, shutil
from django.conf import settings
from django.db import models
from django.apps import apps

import re

pattern = r'^\s*(import\s+[a-zA-Z0-9_., ]+|from\s+[a-zA-Z0-9_.]+\s+import\s+[\*\w., ]+)'

def valid_models():

    retVal = []

    API_GENERATOR = getattr(settings, 'API_GENERATOR')

    for val in API_GENERATOR.values():

        app_name      = val.split('.')[0]
        model_name    = val.split('.')[-1]
        model_import  = val.replace('.'+model_name, '')             

        models = importlib.import_module( model_import )

        try:
            model = getattr(models, model_name)
        except:
            #print(f' > Warn: [' + model_name + '] model NOT_FOUND for [' + app_name + '] APP' )
            #print(f'   Hint: Add [' + model_name + '] model definition in [' + app_name + ']')
            continue 

        try:
            model.objects.last()
        except:
            #print(f' > Warn: [' + model_name + '] model not migrated in DB.' )
            #print(f'   Hint: run makemigrations, migrate commands')
            continue

        #print ( ' Valid API_GEN Model -> ' + val )
        retVal.append( val )

    return retVal

import ast

def extract_top_level_classes(code):
    tree = ast.parse(code)
    classes = []

    # Only look at nodes directly in the module body (top-level)
    for node in tree.body:
        if isinstance(node, ast.ClassDef):
            class_code = ast.get_source_segment(code, node)
            if class_code:
                classes.append(class_code)

    return classes

def file_import_library(file_name, content):
    if(os.path.exists(file_name)):
        with open(file_name, 'r') as f:
            file_content = f.read()
            imp = ''
            matches = re.findall(r'^\s*(import\s+[a-zA-Z0-9_., ]+|from\s+[a-zA-Z0-9_.]+\s+import\s+[\*\w., ]+)', file_content, re.MULTILINE)
            imp += '\n'.join(matches)
            if content in file_content:
                return imp
            return content + imp
    return content

def file_get_classes(file_name):
    cls = ''
    if(os.path.exists(file_name)):
        with open(file_name, 'r') as f:
            file_content = f.read()
            extracted_classes = extract_top_level_classes(file_content)
            if(extracted_classes):
                cls = '\n\n'.join(extracted_classes)
    return cls

def extract_admin_register_calls(content):
    """
    Extract full admin.site.register(...) calls from file content.
    """
    pattern = re.compile(r"admin\.site\.register\s*\((.*?)\)", re.DOTALL)
    return [f"admin.site.register({match.strip()})" for match in pattern.findall(content)]

def file_get_admins(file_name):
    cls = ''
    if(os.path.exists(file_name)):
        with open(file_name, 'r') as f:
            file_content = f.read()
            extracted_admins = extract_admin_register_calls(file_content)
            if(extracted_admins):
                cls = '\n'.join(extracted_admins)
    return cls

def extract_string_fields_from_model(model_file_path, model_name):
    with open(model_file_path, 'r') as file:
        node = ast.parse(file.read())

    string_fields = []

    for class_node in [n for n in node.body if isinstance(n, ast.ClassDef)]:
        if class_node.name == model_name:
            for stmt in class_node.body:
                if isinstance(stmt, ast.Assign) and isinstance(stmt.value, ast.Call):
                    try:
                        field_type = stmt.value.func.attr
                        if field_type in ('CharField', 'TextField'):
                            field_name = stmt.targets[0].id
                            string_fields.append(field_name)
                    except AttributeError:
                        continue
    return string_fields

def extract_foreign_fields_and_targets(model_file_path, model_name):
    with open(model_file_path, 'r') as file:
        node = ast.parse(file.read())

    foreign_fields = []

    for class_node in [n for n in node.body if isinstance(n, ast.ClassDef)]:
        if class_node.name == model_name:
            for stmt in class_node.body:
                if isinstance(stmt, ast.Assign) and isinstance(stmt.value, ast.Call):
                    try:
                        call = stmt.value
                        func = call.func

                        # Must be models.ForeignKey or models.OneToOneField
                        if isinstance(func, ast.Attribute) and func.attr in ('ForeignKey', 'OneToOneField'):
                            field_name = stmt.targets[0].id

                            # Handle first positional arg
                            if call.args:
                                target_arg = call.args[0]
                            else:
                                # Fallback: look for keyword "to"
                                target_arg = next((kw.value for kw in call.keywords if kw.arg == 'to'), None)

                            if isinstance(target_arg, ast.Str):
                                target_model = target_arg.s
                            elif isinstance(target_arg, ast.Name):
                                target_model = target_arg.id
                            elif isinstance(target_arg, ast.Attribute):
                                target_model = f"{target_arg.value.id}.{target_arg.attr}"
                            else:
                                target_model = "Unknown"

                            foreign_fields.append((field_name, target_model))
                    except Exception:
                        continue

    return foreign_fields
def extract_all_fields_from_model(model_file_path, model_name):
    with open(model_file_path, 'r') as file:
        node = ast.parse(file.read())

    all_fields = []

    for class_node in [n for n in node.body if isinstance(n, ast.ClassDef)]:
        if class_node.name == model_name:
            for stmt in class_node.body:
                if isinstance(stmt, ast.Assign) and isinstance(stmt.value, ast.Call):
                    try:
                        field_name = stmt.targets[0].id
                        all_fields.append(field_name)
                    except AttributeError:
                        continue
    return all_fields

def to_kebab_case(s):
    # Replace underscores with spaces
    return to_filename(s).replace("_", "-")

def extract_existing_paths(file_name):
    """
    Extract all lines inside urlpatterns = [ ... ]
    """
    pattern = re.compile(r"urlpatterns\s*=\s*\[\s*(.*?)\s*\]", re.DOTALL)
    s = set()
    if(os.path.exists(file_name)):
        with open(file_name, 'r') as f:
            file_content = f.read()
            match = pattern.search(file_content)
            if match:
                path_block = match.group(1)
                # Split individual lines and strip
                s = set("\t" +line.strip() for line in path_block.splitlines())
    if(s):
        return '\n'.join(s) + "\n"
    return ''

def format_field_ts(field, model_var_name):
    field_name = field.name
    label = str(field.verbose_name).title() if field.verbose_name else field.name.title()
    key = field_name

    if isinstance(field, (models.BooleanField,)):
        return f'''
    {{
      key: "{key}",
      label: "{label}",
      children: {model_var_name}?.{field_name} ? (
        <FaCheck color="green" />
      ) : (
        <FaTimes color="red" />
      ),
    }}'''
    
    elif isinstance(field, (models.DateField, models.DateTimeField)):
        return f'''
    {{
      key: "{key}",
      label: "{label}",
      children: toDateAndTime({model_var_name}?.{field_name}),
    }}'''
    
    elif field.choices:
        enum_name = field.name.title().replace('_', '')
        return f'''
    {{
      key: "{key}",
      label: "{label}",
      children: getEnumName({enum_name}, {model_var_name}?.{field_name}),
    }}'''
    
    elif isinstance(field, models.ForeignKey):
        return f'''
    {{
      key: "{key}",
      label: "{label}",
      children: {model_var_name}?.{field_name}_detail?.name || {model_var_name}?.{field_name},
    }}'''
    
    else:
        return f'''
    {{
      key: "{key}",
      label: "{label}",
      children: {model_var_name}?.{field_name},
    }}'''

def to_camel_case(s):
    # Replace non-alphanumeric characters with space
    s = re.sub(r'[^a-zA-Z0-9]', ' ', s)
    # Split on spaces or existing uppercase word boundaries
    words = re.split(r'\s+|(?<=[a-z])(?=[A-Z])', s)
    # Capitalize and join
    return ''.join(word.capitalize() for word in words if word)

def to_pascal_case(s: str) -> str:
    if "_" in s:
        # snake_case → PascalCase
        return ''.join(word.capitalize() for word in s.split('_'))
    else:
        # camelCase or already PascalCase → PascalCase
        return ''.join(word.capitalize() for word in re.findall(r'[A-Z]?[a-z0-9]+', s))
    
import re

def to_filename(string):
    """
    Converts a camelCase or PascalCase string to lowercase, separates words with underscores,
    removes non-alphanumeric characters (except spaces), and replaces spaces with underscores.
    
    Args:
        string (str): The input string.

    Returns:
        str: The cleaned, underscore-formatted string.
    """
    # Add underscores between camelCase or PascalCase transitions
    string = string.replace('-', ' ')
    string = re.sub(r'(?<=[a-z0-9])(?=[A-Z])', ' ', string)
    string = re.sub(r'[^\w\s]', '', string)        # Remove punctuation
    string = re.sub(r'\s+', '_', string.strip())  # Replace spaces with underscores
    return string.lower()


def to_title_case(string):
    """
    Converts a string to title case.

    Args:
        string (str): The input string.

    Returns:
        str: The title-cased string.
    """
    return string.strip().title()


# -*- encoding: utf-8 -*-
"""
Copyright (c) 2019 - present AppSeed.us
"""

from http import HTTPStatus
from django.http import Http404
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.generics import get_object_or_404
from rest_framework.permissions import IsAuthenticatedOrReadOnly



# -*- encoding: utf-8 -*-
"""
Copyright (c) 2019 - present AppSeed.us
"""

from django.conf import settings
from .util import *
import os

API_GENERATOR = {}

try:
    API_GENERATOR = getattr(settings, 'API_GENERATOR') 
except:     
    pass 

# For cross platform imports 
# https://stackoverflow.com/questions/6028000/how-to-read-a-static-file-from-inside-a-python-package

try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 `importlib_resources`.
    import importlib_resources as pkg_resources

def generate_serializer_file():

    # serializers should be visible in the local namespace
    from . import serializers

    serializers_structure = pkg_resources.read_text(serializers, 'serializers_structure')
    library_imports       = pkg_resources.read_text(serializers, 'library_imports')
    base_serializer       = pkg_resources.read_text(serializers, 'base_serializer')
    foreign_serializers       = pkg_resources.read_text(serializers, 'foreign_serializers')

    #for val in API_GENERATOR.values():
    for val in valid_models():        
        # extract model from import path
        app_name      = val.split('.')[0]
        model_name    = val.split('.')[-1]
        model_import  = val.replace('.'+model_name, '')
        file_name = os.path.join(app_name, 'serializers.py')
        model_file_name = os.path.join(app_name, 'models.py')
        fields = extract_foreign_fields_and_targets(model_file_name, model_name)

        # Build Imports
        project_imports = 'from ' + model_import + ' import ' + model_name

        foreigns = '    '.join([foreign_serializers.format(model_name=field[0], model_class=field[1]) for field in fields])
        
        serializers = base_serializer.format(model_name=model_name, foreigns=foreigns)


        generation = serializers_structure.format(
            library_imports= file_import_library(file_name, library_imports),
            project_imports=project_imports,
            serializers=serializers,
            previous_classes = file_get_classes(file_name),
        )

        with open(file_name, 'w') as serializers_py:
            serializers_py.write(generation)

    return generation

def generate_admin_file():

    # serializers should be visible in the local namespace
    from . import admin

    admin_structure = pkg_resources.read_text(admin, 'admin_structure')
    library_imports       = pkg_resources.read_text(admin, 'library_imports')
    base_admin       = pkg_resources.read_text(admin, 'base_admin')
    base_imports       = pkg_resources.read_text(admin, 'base_imports')


    #for val in API_GENERATOR.values():
    for val in valid_models():        
        # extract model from import path
        app_name      = val.split('.')[0]
        model_name    = val.split('.')[-1]
        file_name = os.path.join(app_name, 'admin.py')

        # Build Imports
        project_imports = base_imports.format(app_name=app_name, model_name=model_name)
        
        admin = base_admin.format(model_name=model_name)

        generation = admin_structure.format(
            library_imports= file_import_library(file_name, library_imports),
            base_imports=project_imports,
            admin=admin,
            previous_admin = file_get_admins(file_name)
        )

        with open(file_name, 'w') as admin_py:
            admin_py.write(generation)

    return generation

def generate_apps_file():

    # serializers should be visible in the local namespace
    from . import apps

    apps_structure = pkg_resources.read_text(admin, 'apps_structure')


    #for val in API_GENERATOR.values():
    for val in valid_models():        
        # extract model from import path
        app_name      = val.split('.')[0]
        model_name    = val.split('.')[-1]
        file_name = os.path.join(app_name, 'apps.py')

        generation = apps_structure.format(
            model_name= model_name,
            model_name_name = to_filename(model_name)
        )

        with open(file_name, 'w') as admin_py:
            admin_py.write(generation)

    return generation

def generate_views_file():

    # views should be visible in the local namespace
    from . import views

    views_structure = pkg_resources.read_text(views, 'views_structure')  
    library_imports = pkg_resources.read_text(views, 'library_imports')
    base_views      = pkg_resources.read_text(views, 'base_views')

    #for val in API_GENERATOR.values():
    for val in valid_models():
        
        # extract model from import path
        app_name      = val.split('.')[0]
        model_name    = val.split('.')[-1]
        model_import  = val.replace('.'+model_name, '') 
        file_name = os.path.join(app_name, 'views.py')
        model_file_name = os.path.join(app_name, 'models.py')
        base_imports    = "from {}.serializers import {}Serializer".format(app_name, model_name)
        # Build Imports
        project_imports = 'from ' + model_import + ' import ' + model_name
        search_fields = extract_string_fields_from_model(model_file_name, model_name)
        views = base_views.format(
            serializer_name=f'{model_name}Serializer',
            model_name=model_name,
            search_fields=search_fields
        )

        generation = views_structure.format(
            library_imports= file_import_library(file_name, library_imports),
            project_imports=project_imports,
            base_imports=base_imports,
            previous_views=file_get_classes(file_name),
            views=views
        )

        with open(file_name, 'w') as views_py:
            views_py.write(generation)

    return generation

def generate_urls_file():

    # urls should be visible in the local namespace
    from . import urls
    
    urls_file_structure = """{library_imports}\n{project_imports}\n\nurlpatterns = [\n{paths}\n]"""

    library_imports = pkg_resources.read_text(urls, 'library_imports') 
    base_imports    = pkg_resources.read_text(urls, 'base_imports')
    base_urls_path  = pkg_resources.read_text(urls, 'base_urls_path')

    #for val in API_GENERATOR.values():
    for endpoint, val in API_GENERATOR.items():
        # extract model from import path
        app_name      = val.split('.')[0]
        model_name    = val.split('.')[-1]
        file_name = os.path.join(app_name, 'urls.py')
        paths = f'{base_urls_path.format(endpoint=endpoint, model_name=model_name, kebab_name=to_kebab_case(model_name))}'

        generation = urls_file_structure.format(
            library_imports= file_import_library(file_name, library_imports),
            project_imports=base_imports.format(app_name=app_name, model_name=model_name),
            paths=extract_existing_paths(file_name) + paths
        )
        with open(file_name, 'w') as urls_py:
            urls_py.write(generation)

    return generation

